%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% CONFIGURATION PART %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Parameters used for mirror matrix computations %%%%%%%%%%%%%%%%%%%%%%%%%%
global rmir npt thr
rmir = 0.6;         % mirror radius in meters
npt = 151;          % number of points for numerical integration on a grid npt x npt
thr = 0;            % threshold to kill small matrix elements (safer to stay at 0!)
c = 299792458;

% Modes parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global mm M
mm = list_modes(4);    % use only even modes for mismatch
M = length(mm);

% Arm cavity parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

itf.RCi = 1416; % input mirror ROC
itf.RCe = 1646;  % end mirror ROC
itf.L   = 3000;  % length

itf.ti = sqrt(0.0069);                  % input mirror  amplitude transmission 
itf.ri = sqrt(0.9931);                  % input mirror amplitude reflection
itf.li = 0;                             % input mirror losses

itf.te = sqrt(75e-6);                   % end mirror amplitude transmission
itf.le = 0;                             % end mirror losses
itf.re = sqrt(1 - itf.le - itf.te^2);   % end mirror amplitude reflection

% compute the arm cavity mode for latter computations
cav = cavity_mode(itf.RCi, itf.RCe, itf.L);

% compute finesse
itf.Finesse = pi * sqrt(itf.ri*itf.re) / (1 - itf.ri * itf.re);

% Recycling cavities parameteres %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

itf.tp = sqrt(0.04);                    % power recycling mirror amplitude transmission
itf.lp = 0;                             % power recycling mirror losses
itf.rp = sqrt(1 - itf.lp - itf.tp^2);   % power recycling mirror amplitude reflection

itf.ts = sqrt(0.11);            % signal recycling mirror amplitude transmission
itf.ls = 35e-6;                 % signal recycling mirror losses
itf.rs = sqrt(1 - itf.ls - itf.ts^2);   % signal recycling mirror amplitude reflection

itf.Lprc = 36.439;               % Julien's design Apr 8th 2010
itf.Lsrc = 35.090;               % Julien's design Apr 8th 2010
itf.Schnupp = 0.10;              % Schnupp asymmetry

itf.fmod1 = 6170368;             % Julien's design Apr 8th 2010
itf.fmod2 = 55533313;            % Julien's design Apr 8th 2010
itf.m = 0.3;

% Define astigmatic recycling cavities (Julien's design Apr 8th 2010)

itf.prec.L1 = (5.889 + 5.789)/2 + 10.450;           % distances between mirrors 
itf.prec.L2 = 10.123;
itf.prec.L3 = 10.027;
itf.prec.R2 = -2.853;                               % radii of curvature
itf.prec.R3 = 22.543;
itf.prec.alpha = 0.9;                               % incidence angle (degrees)

itf.srec.L1 = (5.889 + 5.789)/2 + 9.952;            % distances between mirrors
itf.srec.L2 = 9.652;
itf.srec.L3 = 9.556;
itf.srec.R2 = -2.866;                               % radii of curvature
itf.srec.R3 = 21.626;
itf.srec.alpha = 1.0;                               % incidence angle (degrees)

% Build perfectly aligned recycling mirrors
[R, T, itf.RCp, itf.wp] = ndrc_power_recycling(itf.prec, cav, mm, 0, 0, 0);
itf.Rp = itf.rp * R;
itf.Tp = itf.tp * T;


[R,T, itf.RCs, itf.ws] = ndrc_signal_recycling(itf.srec, cav, mm, 0, 0, 0);;
itf.Rs = itf.rs * R;
itf.Ts = itf.ts * T;

% Gouy phase already considered in model of astigmatic cavities
itf.phig_p = 0;
itf.phig_s = 0;


% Aberrations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
itf.aRCin = itf.RCi;   % radii of curvature
itf.aRCen = itf.RCe;
itf.aRCiw = itf.RCi;   % radii of curvature
itf.aRCew = itf.RCe;
itf.aRCp = itf.RCp;
itf.aRCs = itf.RCs;

itf.fNI = 100e10;      % focal length of input mirror thermal lenses
itf.fWI = 100e10;

% end of parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Build arm cavities (with aberrations if defined)
itf.ncav = build_cavity(itf.RCi, itf.RCe, itf.L, itf.aRCin, itf.aRCen, itf.ri, itf.re, itf.li, itf.le, itf.fNI, mm);
itf.wcav = build_cavity(itf.RCi, itf.RCe, itf.L, itf.aRCiw, itf.aRCew, itf.ri, itf.re, itf.li, itf.le, itf.fWI, mm);

% Length tunings %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
itf.phi_src = 0.15;   
itf.DARM = 1e-11;
itf.PRCL = 0;
itf.CARM = 0;
itf.MICH = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% MISALIGN PRM1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

NPT = 30;
a = linspace(0, 5e-6, NPT);

Crc_prm1 = zeros(M,M,NPT);
Csc_prm1 = zeros(M,M,NPT);
Cdf_prm1 = zeros(M,M,NPT);
Cn_prm1 = zeros(M,M,NPT);
Cw_prm1  = zeros(M,M,NPT);
U1rc_prm1 = zeros(M,M,NPT);
U1sc_prm1 = zeros(M,M,NPT);
U1df_prm1 = zeros(M,M,NPT);
U1n_prm1  = zeros(M,M,NPT);
U1w_prm1  = zeros(M,M,NPT);
L1rc_prm1 = zeros(M,M,NPT);
L1sc_prm1 = zeros(M,M,NPT);
L1df_prm1 = zeros(M,M,NPT);
L1n_prm1  = zeros(M,M,NPT);
L1w_prm1  = zeros(M,M,NPT);
U2rc_prm1 = zeros(M,M,NPT);
U2sc_prm1 = zeros(M,M,NPT);
U2df_prm1 = zeros(M,M,NPT);
U2n_prm1  = zeros(M,M,NPT);
U2w_prm1 = zeros(M,M,NPT);
L2rc_prm1 = zeros(M,M,NPT);
L2sc_prm1 = zeros(M,M,NPT);
L2df_prm1 = zeros(M,M,NPT);
L2n_prm1  = zeros(M,M,NPT);
L2w_prm1  = zeros(M,M,NPT);

tic;
h = waitbar(0, 'Computing');
for i=1:NPT
    % define the recyling cavity with tilted PRM1
    [R, T, itf.RCp, itf.wp] = ndrc_power_recycling(itf.prec, cav, mm, a(i), 0, 0);
    itf.Rp = itf.rp * R;
    itf.Tp = itf.tp * T;
    
    % lock interferometer
    zprcl(i) = fminsearch(@(x) -maximize_sb(itf, x, 0), 0);
    zcarm(i) = fminsearch(@(x) -maximize_car(itf, zprcl(i), x), 0);
    itf.PRCL = 1e-12 * zprcl(i);
    itf.CARM = 1e-12 * zcarm(i);
    zdarm(i) = fzero(@(x) error_darm(itf, x), 10);
    itf.DARM = 1e-12 * zdarm(i);    
    phisr(i) = fminsearch(@(x) -maximize_sens(itf,x), 0.16);
    itf.phi_src = phisr(i);
    
    % compute fields
    [Crc_prm1(:,:,i), Csc_prm1(:,:,i), Cdf_prm1(:,:,i), Cn_prm1(:,:,i), Cw_prm1(:,:,i)] = dualrecycled_fields(itf, 0);
    [U1rc_prm1(:,:,i), U1sc_prm1(:,:,i), U1df_prm1(:,:,i), U1n_prm1(:,:,i), U1w_prm1(:,:,i)] = dualrecycled_fields(itf, +itf.fmod1);
    [L1rc_prm1(:,:,i), L1sc_prm1(:,:,i), L1df_prm1(:,:,i), L1n_prm1(:,:,i), L1w_prm1(:,:,i)] = dualrecycled_fields(itf, -itf.fmod1);
    [U2rc_prm1(:,:,i), U2sc_prm1(:,:,i), U2df_prm1(:,:,i), U2n_prm1(:,:,i), U2w_prm1(:,:,i)] = dualrecycled_fields(itf, +itf.fmod2);
    [L2rc_prm1(:,:,i), L2sc_prm1(:,:,i), L2df_prm1(:,:,i), L2n_prm1(:,:,i), L2w_prm1(:,:,i)] = dualrecycled_fields(itf, -itf.fmod2);
    
    el = toc;
    eta = floor((el / i) * (NPT - i));
    waitbar(i/NPT, h, ['Dual recycled locked (', num2str(floor(100*i/NPT)), '% ETA = ', num2str(eta), 's)']);
end
close(h)

% plots
figure('Position', [0,0,800,600])
semilogy(1e9 * a, 1 - abs(squeeze(Crc_prm1(1,1,:))).^2 / abs(squeeze(Crc_prm1(1,1,1))).^2);
hold all
semilogy(1e9 * a, 1 - abs(squeeze(U1rc_prm1(1,1,:))).^2 / abs(squeeze(U1rc_prm1(1,1,1))).^2);
semilogy(1e9 * a, 1 - abs(squeeze(L1rc_prm1(1,1,:))).^2 / abs(squeeze(L1rc_prm1(1,1,1))).^2);
xlabel('PRM1 tilt [nrad]', 'FontSize', 14)
ylabel('Power loss inside PRC',  'FontSize', 14)
grid
legend('CAR', 'USB', 'LSB')
set(gca, 'FontSize', 14)
title('PRM1 mis-alignment')

