% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
% 
% Compute mutiple lensing matrix, for all focal lenghts in f.
%
% function L = lenses(f, w, modes, rlens, npt, thr)
%
%     f     = vector of focal lenghts
%     w     = beam size
%     modes = list of all modes to be used in [n,m] format
%     rmir  = mirror radius
%     npt   = number of points for numerical integration
%     thr   = below this values coupling coefficients are set to zero

function L = lenses_optimized(f, w, nM, rlens, npt, thr)
    h = waitbar(0, 'Computing thermal lenses (0%)...');
    N = length(f);
    M = length(list_even_modes(nM));
    
    L = zeros(M,M,N);
    
    for i=1:N
        waitbar(i/N, h, ['Computing thermal lenses (', num2str(floor(100*i/N)), '%)']);
        fprintf(2, 'focal = %g (%d/%d)\n', f(i), i, N);
        L(:,:,i) = lens_optimized_even_modes(f(i), w, nM, rlens, npt);
    end
    close(h);
    
