% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
%
% Lock a dual recycled interfemeter using real error signals computed by 
% an auxiliary function error_signals.m
%       
% [zcarm, zprcl, zmich, zdarm, zsrcl] = lockdual(itf, lock, nitermax)
%
%     itf  = interferometer structure
%     lock = structure containing lock parameters 
%         gcarm, gprcl, gmich, gdarm, gsrcl: loop gains
%         tolcarm, toldarm, tolmich, tolprcl, tolsrc: maximum tolerances
%                                                     for error signals
%
%     zcarm, zprcl, zmich, zdarm, zsrcl = final locked positions
%     niter                             = number of iterations perfomed

function [zcarm, zprcl, zmich, zdarm, zsrcl] = lockdual(itf, lock, nitermax)
    
    zdarm = zeros(ntiermax,1);
    zcarm = zeros(ntiermax,1);
    zprcl = zeros(ntiermax,1);
    zmich = zeros(ntiermax,1);
    zsrcl = zeros(ntiermax,1);
    edarm = zeros(ntiermax,1);
    ecarm = zeros(ntiermax,1);
    eprcl = zeros(ntiermax,1);
    emich = zeros(ntiermax,1);
    esrcl = zeros(ntiermax,1);

    % init corrections
    zdarm(1) = itf.DARM;
    zcarm(1) = itf.CARM;
    zmich(1) = itf.MICH;
    zprcl(1) = itf.PRCL;
    zsrcl(1) = itf.phi_src;
    
    % lock
    for i=1:nitermax
        % compute error signals
        itf.DARM = zdarm(i);
        itf.CARM = zcarm(i);
        itf.MICH = zmich(i);
        itf.PRCL = zprcl(i);
        itf.phi_src = zsrcl(i);
        [eprcl(i), ecarm(i), emich(i), edarm(i), esrcl(i)] = error_signals(itf, lock);
        
        % check if lock reached
        locked = abs(eprcl(i) < lock.tolprcl
        
        zdarm(i) = zdarm(i) - edarm(i) * lock.gdarm;
        zcarm(i) = zcarm(i) - ecarm(i) * lock.gcarm;
        zprcl(i) = zprcl(i) - eprcl(i) * lock.gprcl;
        zmich(i) = zmich(i) - emich(i) * lock.gmich;
        zsrcl(i) = zsrcl(i) - esrcl(i) * lock.gsrcl;
        
