% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
%
% Lock a dual recycled interferometer using real error signals. Debug version,
% returns full evolution of locking point and error signals
%
% function [zcarm, zprcl, zmich, zdarm, zsrcl, niter,
%           ecarm, eprcl, emich, edarm, esrcl  ] = lockvirgo_debug(itf, lock, nitermax, g)
%
%     itf  = interferometer structure
%     lock = structure containing lock parameters 
%         gcarm, gprcl, gmich, gdarm, gsrcl: loop gains
%         tolcarm, toldarm, tolmich, tolprcl, tolsrc: maximum tolerances
%                                                     for error signals
%     nitermax = maximum number of iterations
%     g        = loop integrator gain 
%     locksrc  = if 1 lock also SRCL, otherwise keep it fixed
%
%     zcarm, zprcl, zmich, zdarm, zsrcl = final locked positions
%     niter                             = number of iterations perfomed
%


function [zcarm, zprcl, zmich, zdarm, niter, ecarm, eprcl, emich, edarm] = lockvirgo_debug(itf, lock, nitermax, g)
    
    zdarm = zeros(nitermax,1);
    zcarm = zeros(nitermax,1);
    zprcl = zeros(nitermax,1);
    zmich = zeros(nitermax,1);
    edarm = zeros(nitermax,1);
    ecarm = zeros(nitermax,1);
    eprcl = zeros(nitermax,1);
    emich = zeros(nitermax,1);
    
    % init corrections
    zdarm(1) = itf.DARM;
    zcarm(1) = itf.CARM;
    zmich(1) = itf.MICH;
    zprcl(1) = itf.PRCL;
    
    % lock
    for i=1:nitermax
        % compute error signals
        if lock.gdarm ~= 0
            itf.DARM    = zdarm(i);
        end
        if lock.gcarm ~= 0
            itf.CARM    = zcarm(i);
        end
        if lock.gmich ~= 0 
            itf.MICH    = zmich(i);
        end
        if lock.gprcl ~= 0 
            itf.PRCL    = zprcl(i);
        end
        
        [eprcl(i), ecarm(i), emich(i), edarm(i)] = error_signals_virgo(itf, lock);
        
        locked = ((abs(eprcl(i)) < lock.tolprcl ) | (lock.gprcl == 0)) & ...
                 ((abs(ecarm(i)) < lock.tolcarm ) | (lock.gcarm == 0)) & ...
                 ((abs(emich(i)) < lock.tolmich ) | (lock.gmich == 0)) & ...
                 ((abs(edarm(i)) < lock.toldarm ) | (lock.gdarm == 0));
         
        if locked
            break;
        end
        
        if (i == 1)
            if lock.gdarm ~= 0
                zdarm(i+1) = zdarm(i) - edarm(i) * g;
            end
            if lock.gcarm ~= 0
                zcarm(i+1) = zcarm(i) - ecarm(i) * g;
            end
            if lock.gcarm ~= 0
                zprcl(i+1) = zprcl(i) - eprcl(i) * g;
            end
            if lock.gmich ~= 0
                zmich(i+1) = zmich(i) - emich(i) * g;
            end
        else
            if lock.gdarm ~= 0
                zdarm(i+1) = zdarm(i) - (edarm(i) + edarm(i-1)) * g;
            end
            if lock.gcarm ~= 0
                zcarm(i+1) = zcarm(i) - (ecarm(i) + ecarm(i-1)) * g;
            end
            if lock.gprcl ~= 0
                zprcl(i+1) = zprcl(i) - (eprcl(i) + eprcl(i-1)) * g;
            end
            if lock.gmich ~= 0
                zmich(i+1) = zmich(i) - (emich(i) + emich(i-1)) * g;
            end
        end
    end 
 
    niter = i;
    
